import 'bluebird-global';
import sqlite from 'sqlite';
import Emittery from 'emittery';
import { SQLStatement } from 'sql-template-strings';
import { Table, Column, UndefinedEqNull } from './table';
export declare class Db extends Emittery.Typed<{
    'pre-find': {
        cond: any;
        tables: {
            type?: 'inner' | 'left' | 'cross' | 'natural';
            from?: Column;
            cond?: string;
            to: Column | Table<any>;
        }[];
        select: {
            [alias: string]: string | SQLStatement | Column;
        };
        options: {
            postfix?: string;
            sort?: {
                key: Column | string;
                desc?: boolean;
            };
            offset?: number;
            limit?: number;
        };
    };
    'find-sql': SQLStatement;
}> {
    static connect(f: string | sqlite.Database, options?: any): Promise<Db>;
    sql: sqlite.Database;
    private constructor();
    /**
     * Initialize tables sequentially, just in case foreign keys matter
     *
     * @param tables
     */
    init(tables: Table<any>[]): Promise<void>;
    create<E>(table: Table<E>): (entry: UndefinedEqNull<E & {}>, options?: {
        postfix?: SQLStatement | undefined;
        ignoreErrors?: boolean | undefined;
    } | undefined) => Promise<number>;
    each(table0: Table<any>, ...tables: (Table<any> | {
        type?: 'inner' | 'left' | 'cross' | 'natural';
        from?: Column;
        cond?: string;
        to: Column | Table<any>;
    })[]): <Select extends {
        [alias: string]: string | SQLStatement | Column<any>;
    } = Required<{
        [x: string]: Column<any>;
    }>, R = UndefinedEqNull<{ [K in keyof Select]: Select[K] extends Column<infer T> ? T : any; }>>(qCond: Record<string, any>, select: Select | "*", options?: {
        postfix?: string | undefined;
        sort?: {
            key: string | Column<any>;
            desc?: boolean | undefined;
        } | undefined;
        offset?: number | undefined;
        limit?: number | undefined;
    }) => (cb: (r: R) => void) => Promise<number>;
    all(table0: Table<any>, ...tables: (Table<any> | {
        type?: 'inner' | 'left' | 'cross' | 'natural';
        from?: Column;
        cond?: string;
        to: Column | Table<any>;
    })[]): <Select extends {
        [alias: string]: string | SQLStatement | Column<any>;
    } = Required<{
        [x: string]: Column<any>;
    }>>(qCond: Record<string, any>, select: "*" | Select, options?: {
        postfix?: string | undefined;
        sort?: {
            key: string | Column<any>;
            desc?: boolean | undefined;
        } | undefined;
        offset?: number | undefined;
        limit?: number | undefined;
    }) => Promise<UndefinedEqNull<{ [K in keyof Select]: Select[K] extends Column<infer T> ? T : any; }>[]>;
    first(table0: Table<any>, ...tables: (Table<any> | {
        type?: 'inner' | 'left' | 'cross' | 'natural';
        from?: Column;
        cond?: string;
        to: Column | Table<any>;
    })[]): <Select extends {
        [alias: string]: string | SQLStatement | Column<any>;
    } = Required<{
        [x: string]: Column<any>;
    }>>(qCond: Record<string, any>, select: "*" | Select, options?: {
        postfix?: string | undefined;
        sort?: {
            key: string | Column<any>;
            desc?: boolean | undefined;
        } | undefined;
        offset?: number | undefined;
        limit?: number | undefined;
    }) => Promise<UndefinedEqNull<{ [K in keyof Select]: Select[K] extends Column<infer T> ? T : any; }> | null>;
    update(table0: Table<any>, ...tables: (Table<any> | {
        type?: 'inner' | 'left' | 'cross' | 'natural';
        from?: Column;
        cond?: string;
        to: Column | Table<any>;
    })[]): (qCond: Record<string, any>, set: Record<string, any> | {
        table: Table<any, {}, any>;
        set: Record<string, any>;
    }[], options?: {
        postfix?: string | undefined;
        sort?: {
            key: string | Column<any>;
            desc?: boolean | undefined;
        } | undefined;
        offset?: number | undefined;
        limit?: number | undefined;
    }) => Promise<void>;
    delete(table0: Table<any>, ...tables: (Table<any> | {
        type?: 'inner' | 'left' | 'cross' | 'natural';
        from?: Column;
        cond?: string;
        to: Column | Table<any>;
    })[]): (qCond: Record<string, any>, options?: {
        postfix?: string | undefined;
        sort?: {
            key: string | Column<any>;
            desc?: boolean | undefined;
        } | undefined;
        offset?: number | undefined;
        limit?: number | undefined;
    }) => Promise<void>;
    close(): Promise<this>;
    private _find;
    private _findIds;
}
//# sourceMappingURL=db.d.ts.map