"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const sql_template_strings_1 = require("./compat/sql-template-strings");
/**
 *
 * @param q
 * @param cols
 */
function parseCond(q, cols) {
    const subClause = [];
    if (Array.isArray(q.$or)) {
        subClause.push(sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL(q.$or.map((el) => parseCond(el, cols)), ' OR ')})`);
    }
    else if (Array.isArray(q.$and)) {
        subClause.push(sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL(q.$and.map((el) => parseCond(el, cols)), ' AND ')})`);
    }
    else {
        subClause.push(sql_template_strings_1.SQL `(${_parseCondBasic(q, cols)})`);
    }
    if (subClause.length > 1) {
        return sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL(subClause, ' AND ')})`;
    }
    else if (subClause.length === 1) {
        return subClause[0];
    }
    return sql_template_strings_1.SQL `TRUE`;
}
exports.parseCond = parseCond;
function _parseCondBasic(cond, cols) {
    const cList = [];
    function doDefault(k, v) {
        if (strArrayCols.includes(k)) {
            cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} LIKE '%\x1f'||${v}||'\x1f%'`);
        }
        else {
            cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} = ${v}`);
        }
    }
    const strArrayCols = Object.values(cols).map((c) => {
        const strArrayFields = Object.entries(c.m.__meta.prop)
            .filter(([_, v]) => v && v.type === 'StringArray')
            .map(([k]) => k);
        return [
            ...strArrayFields.map((f) => utils_1.safeColumnName(f)),
            ...strArrayFields.map((f) => `${c.m.__meta.name}__${f}`),
        ];
    }).reduce((prev, c) => [...prev, ...c], []);
    for (let [k, v] of Object.entries(cond)) {
        let isPushed = false;
        if (k.includes('.')) {
            const kn = k.split('.');
            k = `json_extract(${utils_1.safeColumnName(kn[0])}, '$.${utils_1.safeColumnName(kn.slice(1).join('.'))}')`;
        }
        else {
            k = utils_1.safeColumnName(k);
        }
        const isStrArray = strArrayCols.includes(k);
        if (v instanceof Date) {
            v = +v;
        }
        if (v) {
            if (Array.isArray(v)) {
                if (isStrArray) {
                    cList.push(sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL((v.map((v0) => {
                        return sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} LIKE '%\x1f'||${v0}||'\x1f%'`;
                    })), ' AND ')})`);
                }
                else {
                    if (v.length > 1) {
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} IN (${sql_template_strings_1.joinSQL(v.map((v0) => sql_template_strings_1.SQL `${v0}`), ',')})`);
                    }
                    else if (v.length === 1) {
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} = ${v[0]}`);
                    }
                }
            }
            else if (typeof v === 'object' && v.constructor === Object) {
                const op = Object.keys(v)[0];
                let v1 = v[op];
                if (v1 instanceof Date) {
                    v1 = +v1;
                }
                if (Array.isArray(v1)) {
                    switch (op) {
                        case '$in':
                            if (isStrArray) {
                                cList.push(sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL(v1.map((v0) => {
                                    return sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} LIKE '%\x1f'||${v0}||'\x1f%'`;
                                }), ' OR ')})`);
                            }
                            else {
                                if (v1.length > 1) {
                                    cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} IN (${sql_template_strings_1.joinSQL(v1.map((v0) => sql_template_strings_1.SQL `${v0}`), ',')})`);
                                }
                                else if (v1.length === 1) {
                                    cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} = ${v1[0]}`);
                                }
                            }
                            isPushed = true;
                            break;
                        case '$nin':
                            if (v1.length > 1) {
                                cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} NOT IN (${sql_template_strings_1.joinSQL(v1.map((v0) => sql_template_strings_1.SQL `${v0}`), ',')})`);
                            }
                            else {
                                cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} != ${v1[0]}`);
                            }
                            isPushed = true;
                            break;
                    }
                }
                if (isPushed) {
                    continue;
                }
                if (v1 && typeof v1 === 'object') {
                    if (v1 instanceof Date) {
                        k = `json_extract(${k}, '$.$milli')`;
                        v1 = +v1;
                    }
                    else {
                        v1 = JSON.stringify(v1);
                    }
                }
                switch (op) {
                    case '$like':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} LIKE ${v1}`);
                        break;
                    case '$nlike':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} NOT LIKE ${v1}`);
                        break;
                    case '$substr':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} LIKE '%'||${v1}||'%'`);
                        break;
                    case '$nsubstr':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} NOT LIKE '%'||${v1}||'%'`);
                        break;
                    case '$exists':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} IS ${sql_template_strings_1.SQL(v1 ? 'NOT NULL' : 'NULL')}`);
                        break;
                    case '$gt':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} > ${v1}`);
                        break;
                    case '$gte':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} >= ${v1}`);
                        break;
                    case '$lt':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} < ${v1}`);
                        break;
                    case '$lte':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} <= ${v1}`);
                        break;
                    case '$ne':
                        cList.push(sql_template_strings_1.SQL `${sql_template_strings_1.SQL(k)} != ${v1}`);
                        break;
                    default:
                        doDefault(k, v);
                }
            }
            else {
                doDefault(k, v);
            }
        }
        else {
            doDefault(k, v);
        }
    }
    if (cList.length > 1) {
        return sql_template_strings_1.SQL `(${sql_template_strings_1.joinSQL(cList, ' AND ')})`;
    }
    else if (cList.length === 1) {
        return cList[0];
    }
    return sql_template_strings_1.SQL `TRUE`;
}
