import sqlite from 'sqlite';
import Emittery from 'emittery';
import { SQLStatement } from 'sql-template-strings';
import { ISqliteMeta, IPropRow, IPrimaryRow } from './decorators';
import { SqliteExt } from './utils';
export declare type UndefinedEqNull<E> = {
    [K in keyof E]: E[K] | (undefined extends E[K] ? null : never);
};
export interface ITransformer<T> {
    get: (repr: any) => T | null;
    set: (data: T) => any;
}
export declare class Column<T = any> {
    opts: {
        name: string;
        table: Table<any>;
        prop?: IPropRow<T> | IPrimaryRow<T>;
    };
    constructor(opts: {
        name: string;
        table: Table<any>;
        prop?: IPropRow<T> | IPrimaryRow<T>;
    });
    get tableName(): any;
    get name(): string;
}
export declare class Table<M = any, AdditionalProps extends {
    ROWID?: number;
    createdAt?: Date;
    updatedAt?: Date;
} = {}, E extends M & AdditionalProps = M & AdditionalProps> extends Emittery.Typed<{
    'build-sql': SQLStatement;
    'pre-create': {
        entry: UndefinedEqNull<E>;
        options: {
            postfix: SQLStatement[];
        };
    };
    'create-sql': SQLStatement;
    'pre-update': {
        sql: SQLStatement;
        set: Partial<UndefinedEqNull<E>>;
    };
    'update-sql': SQLStatement;
    'pre-delete': {
        sql: SQLStatement;
    };
    'delete-sql': SQLStatement;
}> {
    c: Required<{
        [K in keyof E]: Column<E[K]>;
    }>;
    m: E & {
        __meta: ISqliteMeta<E>;
    };
    get primaryKey(): string;
    get name(): string;
    constructor(M: {
        new (): M;
    });
    __init(db: sqlite.Database): Promise<void>;
    create(db: sqlite.Database): (entry: UndefinedEqNull<E>, options?: {
        postfix?: SQLStatement;
        ignoreErrors?: boolean;
    }) => Promise<number>;
    __updateBySql(db: sqlite.Database): (sql: SQLStatement, set: Partial<E>) => Promise<void>;
    __deleteBySql(db: sqlite.Database): (sql: SQLStatement) => Promise<void>;
    /**
     * @internal
     * @param k
     * @param method
     */
    transform(k: string, method?: 'get' | 'set'): (a: any) => any;
}
export declare const _transformers: Record<SqliteExt, ITransformer<any>>;
//# sourceMappingURL=table.d.ts.map