"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sql_template_strings_1 = require("./compat/sql-template-strings");
exports.AliasToSqliteType = {
    /**
     * Identity types
     */
    TEXT: sql_template_strings_1.SQL `TEXT`,
    INTEGER: sql_template_strings_1.SQL `INTEGER`,
    REAL: sql_template_strings_1.SQL `REAL`,
    BLOB: sql_template_strings_1.SQL `BLOB`,
    /**
     * Class name types
     */
    String: sql_template_strings_1.SQL `TEXT`,
    Number: sql_template_strings_1.SQL `REAL`,
    Date: sql_template_strings_1.SQL `INTEGER`,
    ArrayBuffer: sql_template_strings_1.SQL `BLOB`,
    Boolean: sql_template_strings_1.SQL `INTEGER`,
    Object: sql_template_strings_1.SQL `TEXT`,
    /**
     * Additional aliases
     */
    JSON: sql_template_strings_1.SQL `TEXT`,
    StringArray: sql_template_strings_1.SQL `TEXT`,
    str: sql_template_strings_1.SQL `TEXT`,
    string: sql_template_strings_1.SQL `TEXT`,
    int: sql_template_strings_1.SQL `INTEGER`,
    integer: sql_template_strings_1.SQL `INTEGER`,
    float: sql_template_strings_1.SQL `REAL`,
    bin: sql_template_strings_1.SQL `BLOB`,
    binary: sql_template_strings_1.SQL `BLOB`,
    boolean: sql_template_strings_1.SQL `INTEGER`,
    bool: sql_template_strings_1.SQL `INTEGER`,
};
const normalizer = `
/**
 * Class name types
 */
String: string // TEXT
Number: number // REAL
Date: Date
ArrayBuffer: ArrayBuffer // BLOB
Boolean: boolean
Object: Record<string, any> | any[] // JSON
/**
 * Additional aliases
 */
JSON: Record<string, any> | any[]
StringArray: string[]
str: string // TEXT
string: string // TEXT
int: number // INTEGER
integer: number // INTEGER
float: number // REAL
bin: ArrayBuffer // BLOB
binary: ArrayBuffer // BLOB
boolean: boolean // Boolean
bool: boolean // Boolean
  `.split('\n')
    .map((r) => r.trim())
    .map((r) => {
    if (r) {
        const m = /^(.+?):.+?\/\/ (.+)$/.exec(r);
        if (m) {
            return [m[1], m[2]];
        }
    }
    return null;
})
    .filter((r) => r)
    .map((r) => r)
    .reduce((prev, [k, v]) => ({ ...prev, [k]: v }), {});
function normalizeAlias(k) {
    return normalizer[k] || k;
}
exports.normalizeAlias = normalizeAlias;
function isNullOrUndefined(a) {
    if (a === null || a === undefined) {
        return true;
    }
    return false;
}
exports.isNullOrUndefined = isNullOrUndefined;
/**
 * https://www.sqlite.org/lang_keywords.html
 * @param s identifier
 */
function safeColumnName(s) {
    const keywords = `
    ABORT
    ACTION
    ADD
    AFTER
    ALL
    ALTER
    ALWAYS
    ANALYZE
    AND
    AS
    ASC
    ATTACH
    AUTOINCREMENT
    BEFORE
    BEGIN
    BETWEEN
    BY
    CASCADE
    CASE
    CAST
    CHECK
    COLLATE
    COLUMN
    COMMIT
    CONFLICT
    CONSTRAINT
    CREATE
    CROSS
    CURRENT
    CURRENT_DATE
    CURRENT_TIME
    CURRENT_TIMESTAMP
    DATABASE
    DEFAULT
    DEFERRABLE
    DEFERRED
    DELETE
    DESC
    DETACH
    DISTINCT
    DO
    DROP
    EACH
    ELSE
    END
    ESCAPE
    EXCEPT
    EXCLUDE
    EXCLUSIVE
    EXISTS
    EXPLAIN
    FAIL
    FILTER
    FIRST
    FOLLOWING
    FOR
    FOREIGN
    FROM
    FULL
    GENERATED
    GLOB
    GROUP
    GROUPS
    HAVING
    IF
    IGNORE
    IMMEDIATE
    IN
    INDEX
    INDEXED
    INITIALLY
    INNER
    INSERT
    INSTEAD
    INTERSECT
    INTO
    IS
    ISNULL
    JOIN
    KEY
    LAST
    LEFT
    LIKE
    LIMIT
    MATCH
    NATURAL
    NO
    NOT
    NOTHING
    NOTNULL
    NULL
    NULLS
    OF
    OFFSET
    ON
    OR
    ORDER
    OTHERS
    OUTER
    OVER
    PARTITION
    PLAN
    PRAGMA
    PRECEDING
    PRIMARY
    QUERY
    RAISE
    RANGE
    RECURSIVE
    REFERENCES
    REGEXP
    REINDEX
    RELEASE
    RENAME
    REPLACE
    RESTRICT
    RIGHT
    ROLLBACK
    ROW
    ROWS
    SAVEPOINT
    SELECT
    SET
    TABLE
    TEMP
    TEMPORARY
    THEN
    TIES
    TO
    TRANSACTION
    TRIGGER
    UNBOUNDED
    UNION
    UNIQUE
    UPDATE
    USING
    VACUUM
    VALUES
    VIEW
    VIRTUAL
    WHEN
    WHERE
    WINDOW
    WITH
    WITHOUT`
        .split('\n')
        .map((el) => el.trim())
        .filter((el) => el);
    /**
     * https://stackoverflow.com/questions/31788990/sqlite-what-are-the-restricted-characters-for-identifiers
     */
    const validIdToken = 'A-Z0-9_$:';
    const kwRegex = new RegExp(`(^|[^${validIdToken}\\)])(${keywords.join('|')})($|[^${validIdToken}\\()])`, 'gi');
    return s.replace(kwRegex, '$1"$2"$3');
}
exports.safeColumnName = safeColumnName;
